# Copyright 2013-2015 VPAC
# Copyright 2014 The University of Melbourne
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import six

import tldap.schemas as schemas
import tldap.schemas.ad
import tldap.schemas.rfc
import tldap.schemas.ds389
import tldap.methods as methods
import tldap.methods.ad
import tldap.methods.common
import tldap.methods.pwdpolicy
import tldap.methods.ds389
import tldap.manager


def _a(string):
    """ Force string to ASCII only characters. """
    string = ''.join(c for c in string if 31 < ord(c) < 127)
    return string


class PersonMixin(object):

    @classmethod
    def pre_save(cls, self):
        full_name = getattr(self, "fullName", None)
        if full_name is None:
            full_name = "%s %s" % (self.givenName, self.sn)

        self.displayName = six.u('%s (%s)') % (full_name, self.o)


class AccountMixin(object):

    @classmethod
    def pre_save(cls, self):
        full_name = getattr(self, "fullName", None)
        if full_name is None:
            full_name = "%s %s" % (self.givenName, self.sn)

        self.displayName = six.u('%s (%s)') % (full_name, self.o)
        self.gecos = _a(six.u('%s (%s)') % (full_name, self.o))


############
# OpenLDAP #
############

class openldap_person(methods.baseMixin):

    schema_list = [
        schemas.rfc.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.rfc.pwdPolicy,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.pwdpolicy.pwdPolicyMixin,
        PersonMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_PERSON_BASE"
        object_classes = {'top'}
        search_classes = {'person'}
        pk = 'uid'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.openldap_person',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.openldap_person',
        linked_key='manager')


class openldap_account(methods.baseMixin):

    schema_list = [
        schemas.rfc.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.rfc.posixAccount,
        schemas.rfc.shadowAccount,
        schemas.rfc.pwdPolicy,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.common.accountMixin,
        methods.pwdpolicy.pwdPolicyMixin,
        AccountMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_ACCOUNT_BASE"
        object_classes = {'top'}
        search_classes = {'posixAccount'}
        pk = 'uid'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.openldap_account',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.openldap_account',
        linked_key='manager')
    unixHomeDirectory = tldap.manager.AliasDescriptor("homeDirectory")


class openldap_kg27(methods.baseMixin):

    schema_list = [
        schemas.rfc.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.rfc.posixAccount,
        schemas.rfc.shadowAccount,
        schemas.rfc.pwdPolicy,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.common.accountMixin,
        methods.pwdpolicy.pwdPolicyMixin,
        AccountMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_ACCOUNT_BASE"
        object_classes = {'top'}
        # this will find people and accounts
        search_classes = {'person'}
        pk = 'uid'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.openldap_kg27',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.openldap_kg27',
        linked_key='manager')
    unixHomeDirectory = tldap.manager.AliasDescriptor("homeDirectory")


class openldap_person_group(methods.baseMixin):

    schema_list = [schemas.rfc.posixGroup]
    mixin_list = [methods.common.groupMixin]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'posixGroup'}
        pk = 'cn'

    # people
    secondary_people = tldap.manager.ManyToManyDescriptor(
        this_key='memberUid',
        linked_cls=openldap_person, linked_key='uid', linked_is_p=False,
        related_name="secondary_groups")


class openldap_account_group(methods.baseMixin):

    schema_list = [schemas.rfc.posixGroup]
    mixin_list = [methods.common.groupMixin]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'posixGroup'}
        pk = 'cn'

    # accounts
    primary_accounts = tldap.manager.OneToManyDescriptor(
        this_key='gidNumber',
        linked_cls=openldap_account, linked_key='gidNumber',
        related_name="primary_group")
    secondary_accounts = tldap.manager.ManyToManyDescriptor(
        this_key='memberUid',
        linked_cls=openldap_account, linked_key='uid', linked_is_p=False,
        related_name="secondary_groups")


class openldap_kg27_group(methods.baseMixin):

    schema_list = [schemas.rfc.posixGroup]
    mixin_list = [methods.common.groupMixin]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'posixGroup'}
        pk = 'cn'

    # accounts
    primary_accounts = tldap.manager.OneToManyDescriptor(
        this_key='gidNumber',
        linked_cls=openldap_kg27, linked_key='gidNumber',
        related_name="primary_group")
    secondary_accounts = tldap.manager.ManyToManyDescriptor(
        this_key='memberUid',
        linked_cls=openldap_kg27, linked_key='uid', linked_is_p=False,
        related_name="secondary_groups")


############
# 389 LDAP #
############

class ds389AccountMixin(object):

    @classmethod
    def pre_save(cls, self):
        # work around for https://bugzilla.redhat.com/show_bug.cgi?id=1171308
        if self.userPassword is None:
            from karaage.people.models import Person
            self.change_password(Person.objects.make_random_password())


class ds389_person(methods.baseMixin):

    schema_list = [
        schemas.rfc.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.ds389.passwordObject,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.ds389.passwordObjectMixin,
        PersonMixin,
        ds389AccountMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_PERSON_BASE"
        object_classes = {'top'}
        search_classes = {'person'}
        pk = 'uid'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.ds389_person',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.ds389_person',
        linked_key='manager')


class ds389_account(methods.baseMixin):

    schema_list = [
        schemas.rfc.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.rfc.posixAccount,
        schemas.rfc.shadowAccount,
        schemas.ds389.passwordObject,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.common.accountMixin,
        methods.ds389.passwordObjectMixin,
        AccountMixin,
        ds389AccountMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_ACCOUNT_BASE"
        object_classes = {'top'}
        search_classes = {'posixAccount'}
        pk = 'uid'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.ds389_account',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.ds389_account',
        linked_key='manager')
    unixHomeDirectory = tldap.manager.AliasDescriptor("homeDirectory")


class ds389_kg27(methods.baseMixin):

    schema_list = [
        schemas.rfc.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.rfc.posixAccount,
        schemas.rfc.shadowAccount,
        schemas.ds389.passwordObject,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.common.accountMixin,
        methods.ds389.passwordObjectMixin,
        AccountMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_ACCOUNT_BASE"
        object_classes = {'top'}
        search_classes = {'person'}
        pk = 'uid'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.ds389_kg27',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.ds389_kg27',
        linked_key='manager')
    unixHomeDirectory = tldap.manager.AliasDescriptor("homeDirectory")


class ds389_person_group(methods.baseMixin):

    schema_list = [schemas.rfc.posixGroup]
    mixin_list = [methods.common.groupMixin]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'posixGroup'}
        pk = 'cn'

    # people
    secondary_people = tldap.manager.ManyToManyDescriptor(
        this_key='memberUid',
        linked_cls=ds389_person, linked_key='uid', linked_is_p=False,
        related_name="secondary_groups")


class ds389_account_group(methods.baseMixin):

    schema_list = [schemas.rfc.posixGroup]
    mixin_list = [methods.common.groupMixin]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'posixGroup'}
        pk = 'cn'

    # accounts
    primary_accounts = tldap.manager.OneToManyDescriptor(
        this_key='gidNumber',
        linked_cls=ds389_account, linked_key='gidNumber',
        related_name="primary_group")
    secondary_accounts = tldap.manager.ManyToManyDescriptor(
        this_key='memberUid',
        linked_cls=ds389_account, linked_key='uid', linked_is_p=False,
        related_name="secondary_groups")


class ds389_kg27_group(methods.baseMixin):

    schema_list = [schemas.rfc.posixGroup]
    mixin_list = [methods.common.groupMixin]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'posixGroup'}
        pk = 'cn'

    # accounts
    primary_accounts = tldap.manager.OneToManyDescriptor(
        this_key='gidNumber',
        linked_cls=ds389_kg27, linked_key='gidNumber',
        related_name="primary_group")
    secondary_accounts = tldap.manager.ManyToManyDescriptor(
        this_key='memberUid',
        linked_cls=ds389_kg27, linked_key='uid', linked_is_p=False,
        related_name="secondary_groups")


####################
# Active Directory #
####################

class ad_person(methods.baseMixin):

    schema_list = [
        schemas.ad.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.ad.user,
        schemas.ad.posixAccount,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.common.accountMixin,
        methods.ad.adUserMixin,
        PersonMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_PERSON_BASE"
        object_classes = {'top'}
        search_classes = {'user'}
        pk = 'cn'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.ad_person',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.ad_person',
        linked_key='manager')


class ad_account(methods.baseMixin):

    schema_list = [
        schemas.ad.person,
        schemas.rfc.organizationalPerson,
        schemas.rfc.inetOrgPerson,
        schemas.ad.user,
        schemas.ad.posixAccount,
    ]

    mixin_list = [
        methods.common.personMixin,
        methods.common.accountMixin,
        methods.ad.adUserMixin,
        AccountMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_ACCOUNT_BASE"
        object_classes = {'top'}
        search_classes = {'user'}
        pk = 'cn'

    managed_by = tldap.manager.ManyToOneDescriptor(
        this_key='manager',
        linked_cls='karaage.datastores.ldap_schemas.ad_account',
        linked_key='dn')
    manager_of = tldap.manager.OneToManyDescriptor(
        this_key='dn',
        linked_cls='karaage.datastores.ldap_schemas.ad_account',
        linked_key='manager')


class ad_person_group(methods.baseMixin):

    schema_list = [
        schemas.ad.group,
    ]

    mixin_list = [
        methods.ad.adGroupMixin,
    ]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'group'}
        pk = 'cn'

    # accounts
    secondary_people = tldap.manager.AdAccountLinkDescriptor(
        linked_cls=ad_person, related_name="secondary_groups")


class ad_account_group(methods.baseMixin):

    schema_list = [
        schemas.rfc.posixGroup,
        schemas.ad.group,
    ]

    mixin_list = [
        methods.common.groupMixin,
        methods.ad.adGroupMixin
    ]

    class Meta:
        base_dn_setting = "LDAP_GROUP_BASE"
        object_classes = {'top'}
        search_classes = {'group'}
        pk = 'cn'

    # accounts
    primary_accounts = tldap.manager.OneToManyDescriptor(
        this_key='gidNumber',
        linked_cls=ad_account, linked_key='gidNumber',
        related_name="primary_group")
    secondary_accounts = tldap.manager.AdAccountLinkDescriptor(
        linked_cls=ad_account, related_name="secondary_groups")



# Copyright 2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import six
import datetime

import django_tables2 as tables

from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.template.defaultfilters import wordwrap
from django.apps import apps

from karaage.common.decorators import admin_required, login_required
from karaage.people.models import Person
from karaage.common import log
import karaage.common as util

from .models import SoftwareCategory, Software, SoftwareVersion
from .models import SoftwareLicense, SoftwareLicenseAgreement
from .forms import SoftwareForm, AddPackageForm, LicenseForm
from .forms import SoftwareVersionForm, SoftwareCategoryForm
from .tables import SoftwareFilter, SoftwareTable


if apps.is_installed("karaage.plugins.kgsoftware.applications"):
    from karaage.plugins.kgapplications.tables import ApplicationTable
    from .applications.views import new_application
    from .applications.models import SoftwareApplication

    def is_application_pending(person, software_license):
        query = SoftwareApplication.objects.get_for_applicant(person)
        query = query.filter(software_license=software_license)
        if query.count() > 0:
            return True

    def get_applications(software_license):
        applications = SoftwareApplication.objects.filter(
            software_license=software_license)
        applications = applications.exclude(state='C')
        return applications

    def get_applications_for_person(person, software_license):
        applications = SoftwareApplication.objects.get_for_applicant(person)
        applications = applications.filter(software_license=software_license)
        return applications

    def get_application_table(request, applications):
        applications_table = ApplicationTable(applications)
        config = tables.RequestConfig(request, paginate={"per_page": 5})
        config.configure(applications_table)
        return applications_table

else:
    from django.http import HttpResponseBadRequest

    def is_application_pending(person, software_license):
        return False

    def get_applications(software_license):
        return []

    def get_applications_for_person(person, software_license):
        return []

    def get_application_table(request, applications):
        return None

    @login_required
    def new_application(request, software_license):
        return HttpResponseBadRequest("<h1>Restricted Software denied.</h1>")


@login_required
def profile_software(request):
    person = request.user
    agreement_list = person.softwarelicenseagreement_set.all()
    return render_to_response(
        'kgsoftware/profile_software.html',
        {'person': person, 'agreement_list': agreement_list},
        context_instance=RequestContext(request))


@login_required
def software_list(request):
    if not util.is_admin(request):
        return _software_list_non_admin(request)

    queryset = Software.objects.all().select_related()
    q_filter = SoftwareFilter(request.GET, queryset=queryset)
    table = SoftwareTable(q_filter.qs)
    tables.RequestConfig(request).configure(table)

    spec = []
    for name, value in six.iteritems(q_filter.form.cleaned_data):
        if value is not None and value != "":
            name = name.replace('_', ' ').capitalize()
            spec.append((name, value))

    return render_to_response(
        'kgsoftware/software_list.html',
        {
            'table': table,
            'filter': q_filter,
            'spec': spec,
            'title': "Software list",
        },
        context_instance=RequestContext(request))


@login_required
def _software_list_non_admin(request):

    person = request.user

    query = Software.objects.filter(softwarelicense__isnull=False).distinct()
    software_list = []
    for software in query:
        data = {'software': software}
        license_agreements = SoftwareLicenseAgreement.objects.filter(
            person=person, license__software=software)
        if license_agreements.count() > 0:
            la = license_agreements.latest()
            data['accepted'] = True
            data['accepted_date'] = la.date

        software_license = software.get_current_license()
        data['pending'] = is_application_pending(person, software_license)
        software_list.append(data)

    return render_to_response(
        'kgsoftware/add_package_list.html',
        locals(),
        context_instance=RequestContext(request))


@login_required
def software_detail(request, software_id):
    software = get_object_or_404(Software, pk=software_id)
    software_license = software.get_current_license()
    person = request.user
    license_agreements = SoftwareLicenseAgreement.objects \
        .filter(person=person, license=software_license)
    agreement = None
    if license_agreements.count() > 0:
        agreement = license_agreements.latest()

    # we only list applications for current software license

    applications = get_applications(software_license)
    application_table = get_application_table(request, applications)
    open_applications = get_applications_for_person(person, software_license)

    if agreement is None and software_license is not None \
            and len(open_applications) == 0 and request.method == 'POST':

        if software.restricted and not util.is_admin(request):
            log.add(software, "New application created for %s" % request.user)
            return new_application(request, software_license)

        SoftwareLicenseAgreement.objects.create(
            person=person,
            license=software_license,
            date=datetime.datetime.today(),
        )
        person.add_group(software.group)
        log.add(
            software,
            "Approved join (not restricted) for %s" % request.user)
        messages.success(request, "Approved access to %s." % software)
        return HttpResponseRedirect(reverse('kg_profile_software'))

    return render_to_response(
        'kgsoftware/software_detail.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def add_package(request):
    if request.method == 'POST':
        form = AddPackageForm(request.POST)

        if form.is_valid():
            software = form.save()
            return HttpResponseRedirect(software.get_absolute_url())
    else:
        form = AddPackageForm()

    return render_to_response(
        'kgsoftware/add_package_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def software_edit(request, software_id):
    from karaage.common.create_update import update_object
    return update_object(
        request, object_id=software_id, model=Software,
        form_class=SoftwareForm)


@admin_required
def software_delete(request, software_id):
    from karaage.common.create_update import delete_object
    return delete_object(
        request, post_delete_redirect=reverse('kg_software_list'),
        object_id=software_id, model=Software)


@admin_required
def software_logs(request, software_id):
    obj = get_object_or_404(Software, pk=software_id)
    breadcrumbs = [
        ("Softwares", reverse("kg_software_list")),
        (six.text_type(obj), reverse("kg_software_detail", args=[obj.pk]))
    ]
    return util.log_list(request, breadcrumbs, obj)


@admin_required
def add_comment(request, software_id):
    obj = get_object_or_404(Software, pk=software_id)
    breadcrumbs = [
        ("Softwares", reverse("kg_software_list")),
        (six.text_type(obj), reverse("kg_software_detail", args=[obj.pk]))
    ]
    return util.add_comment(request, breadcrumbs, obj)


@login_required
def license_detail(request, license_id):
    l = get_object_or_404(SoftwareLicense, pk=license_id)
    return render_to_response(
        'kgsoftware/license_detail.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def add_license(request, software_id):
    software = get_object_or_404(Software, pk=software_id)

    form = LicenseForm(request.POST or None)
    if request.method == 'POST':
        if form.is_valid():
            l = form.save()
            log.add(software, "license: %s added" % l)
            return HttpResponseRedirect(software.get_absolute_url())

    return render_to_response(
        'kgsoftware/license_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def edit_license(request, license_id):
    l = get_object_or_404(SoftwareLicense, pk=license_id)
    software = l.software

    form = LicenseForm(request.POST or None, instance=l)
    if request.method == 'POST':
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(software.get_absolute_url())

    return render_to_response(
        'kgsoftware/license_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def license_delete(request, license_id):
    from karaage.common.create_update import delete_object
    return delete_object(
        request,
        post_delete_redirect=reverse('kg_software_list'),
        object_id=license_id, model=SoftwareLicense)


@admin_required
def delete_version(request, version_id):
    version = get_object_or_404(SoftwareVersion, pk=version_id)

    if request.method == 'POST':
        version.delete()
        log.delete(version.software, 'Deleted version: %s' % version)

        messages.success(
            request, "Version '%s' was deleted succesfully" % version)
        return HttpResponseRedirect(version.get_absolute_url())

    return render_to_response(
        'kgsoftware/version_confirm_delete.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def add_version(request, software_id):
    software = get_object_or_404(Software, pk=software_id)

    form = SoftwareVersionForm(request.POST or None)
    if request.method == 'POST':
        if form.is_valid():
            version = form.save()
            return HttpResponseRedirect(software.get_absolute_url())

    return render_to_response(
        'kgsoftware/version_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def edit_version(request, version_id):
    version = get_object_or_404(SoftwareVersion, pk=version_id)
    software = version.software

    form = SoftwareVersionForm(request.POST or None, instance=version)
    if request.method == 'POST':
        if form.is_valid():
            version = form.save()
            return HttpResponseRedirect(software.get_absolute_url())

    return render_to_response(
        'kgsoftware/version_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def category_list(request):
    category_list = SoftwareCategory.objects.all()
    return render_to_response(
        'kgsoftware/category_list.html',
        {'category_list': category_list},
        context_instance=RequestContext(request))


@admin_required
def category_create(request):
    from karaage.common.create_update import create_object
    return create_object(
        request, model=SoftwareCategory,
        form_class=SoftwareCategoryForm)


@admin_required
def category_edit(request, category_id):
    from karaage.common.create_update import update_object
    return update_object(
        request, object_id=category_id, model=SoftwareCategory,
        form_class=SoftwareCategoryForm)


@admin_required
def remove_member(request, software_id, person_id):
    software = get_object_or_404(Software, pk=software_id)
    person = get_object_or_404(Person, pk=person_id)

    if request.method == 'POST':
        person.remove_group(software.group)
        messages.success(request, "User '%s' removed successfuly" % person)

        return HttpResponseRedirect(software.get_absolute_url())

    return render_to_response(
        'kgsoftware/person_confirm_remove.html',
        locals(),
        context_instance=RequestContext(request))


@login_required
def license_txt(request, software_id):

    software = get_object_or_404(Software, pk=software_id)
    software_license = software.get_current_license()

    if software_license is None:
        raise Http404('No license found for software')

    return HttpResponse(
        wordwrap(software_license.text, 80),
        content_type="text/plain")



# Copyright 2008, 2010-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import six
import django_tables2 as tables

from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404, render_to_response
from django.http import HttpResponseRedirect
from django.template import RequestContext

from karaage.projects.tables import ProjectTable
from karaage.projects.models import Project
from karaage.machines.tables import MachineTable, MachineCategoryTable
from karaage.machines.tables import AccountTable
from karaage.machines.models import Machine, MachineCategory
from karaage.machines.forms import MachineForm, MachineCategoryForm
from karaage.common.decorators import admin_required, login_required
import karaage.common as util


@login_required
def machine_detail(request, machine_id):
    machine = get_object_or_404(Machine, pk=machine_id)
    return render_to_response(
        'karaage/machines/machine_detail.html',
        {'machine': machine},
        context_instance=RequestContext(request))


@admin_required
def machine_create(request):
    form = MachineForm(request.POST or None)
    if request.method == 'POST':
        if form.is_valid():
            machine = form.save()
            return HttpResponseRedirect(machine.get_absolute_url())

    return render_to_response(
        'karaage/machines/machine_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def machine_edit(request, machine_id):
    machine = get_object_or_404(Machine, pk=machine_id)

    form = MachineForm(request.POST or None, instance=machine)
    if request.method == 'POST':
        if form.is_valid():
            machine = form.save()
            return HttpResponseRedirect(machine.get_absolute_url())

    return render_to_response(
        'karaage/machines/machine_form.html',
        locals(),
        context_instance=RequestContext(request))


@admin_required
def machine_password(request, machine_id):
    machine = get_object_or_404(Machine, pk=machine_id)
    password = None
    if request.method == 'POST':
        password = Machine.objects.make_random_password()
        machine.set_password(password)
        machine.save()
    return render_to_response(
        'karaage/machines/machine_password.html',
        {'machine': machine, 'password': password},
        context_instance=RequestContext(request))


@admin_required
def machine_logs(request, machine_id):
    obj = get_object_or_404(Machine, pk=machine_id)
    breadcrumbs = [
        ("Machines", reverse("kg_machine_category_list")),
        (
            six.text_type(obj.category),
            reverse("kg_machine_category_detail", args=[obj.category.pk])
        ),
        (six.text_type(obj), reverse("kg_machine_detail", args=[obj.pk]))
    ]
    return util.log_list(request, breadcrumbs, obj)


@admin_required
def machine_add_comment(request, machine_id):
    obj = get_object_or_404(Machine, pk=machine_id)
    breadcrumbs = [
        ("Machines", reverse("kg_machine_category_list")),
        (
            six.text_type(obj.category),
            reverse("kg_machine_category_detail", args=[obj.category.pk])
        ),
        (six.text_type(obj), reverse("kg_machine_detail", args=[obj.pk]))
    ]
    return util.add_comment(request, breadcrumbs, obj)


@login_required
def category_list(request):
    queryset = MachineCategory.objects.all()

    table = MachineCategoryTable(queryset)
    tables.RequestConfig(request).configure(table)

    return render_to_response(
        'karaage/machines/machinecategory_list.html',
        {'table': table},
        context_instance=RequestContext(request))


@admin_required
def category_create(request):
    from karaage.common.create_update import create_object
    return create_object(
        request, model=MachineCategory,
        form_class=MachineCategoryForm,
        template_name="karaage/machines/machinecategory_form.html")


@admin_required
def category_edit(request, category_id):
    from karaage.common.create_update import update_object
    return update_object(
        request, object_id=category_id, model=MachineCategory,
        form_class=MachineCategoryForm,
        template_name="karaage/machines/machinecategory_form.html")


@login_required
def category_detail(request, category_id):
    machine_category = get_object_or_404(MachineCategory, pk=category_id)

    queryset = machine_category.machine_set.all()
    table = MachineTable(queryset)
    tables.RequestConfig(request).configure(table)

    return render_to_response(
        'karaage/machines/machinecategory_detail.html',
        {'machine_category': machine_category, 'table': table},
        context_instance=RequestContext(request))


@admin_required
def category_accounts(request, category_id):
    machine_category = get_object_or_404(MachineCategory, pk=category_id)

    queryset = machine_category.account_set.all()
    table = AccountTable(queryset)
    tables.RequestConfig(request).configure(table)

    return render_to_response(
        'karaage/machines/machinecategory_accounts.html',
        {'machine_category': machine_category, 'table': table},
        context_instance=RequestContext(request))


@admin_required
def category_projects(request, category_id):
    machine_category = get_object_or_404(MachineCategory, pk=category_id)

    queryset = Project.objects.filter(
        projectquota__machine_category=machine_category)
    table = ProjectTable(queryset)
    tables.RequestConfig(request).configure(table)

    return render_to_response(
        'karaage/machines/machinecategory_projects.html',
        {'machine_category': machine_category, 'table': table},
        context_instance=RequestContext(request))


@admin_required
def category_logs(request, category_id):
    obj = get_object_or_404(MachineCategory, pk=category_id)
    breadcrumbs = [
        ("Machines", reverse("kg_machine_category_list")),
        (
            six.text_type(obj),
            reverse("kg_machine_category_detail", args=[obj.pk])
        )
    ]
    return util.log_list(request, breadcrumbs, obj)


@admin_required
def category_add_comment(request, category_id):
    obj = get_object_or_404(MachineCategory, pk=category_id)
    breadcrumbs = [
        ("Machines", reverse("kg_machine_category_list")),
        (
            six.text_type(obj),
            reverse("kg_machine_category_detail", args=[obj.pk])
        )
    ]
    return util.add_comment(request, breadcrumbs, obj)



# encoding: utf-8
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SoftwareCategory'
        db.create_table('software_category', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal('software', ['SoftwareCategory'])

        # Adding model 'SoftwarePackage'
        db.create_table('software_package', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['software.SoftwareCategory'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('gid', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('homepage', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('tutorial_url', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('academic_only', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
        ))
        db.send_create_signal('software', ['SoftwarePackage'])

        # Adding model 'SoftwareVersion'
        db.create_table('software_version', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('package', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['software.SoftwarePackage'])),
            ('version', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('module', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
        ))
        db.send_create_signal('software', ['SoftwareVersion'])

        # Adding M2M table for field machines on 'SoftwareVersion'
        db.create_table('software_version_machines', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('softwareversion', models.ForeignKey(orm['software.softwareversion'], null=False)),
            ('machine', models.ForeignKey(orm['machines.machine'], null=False))
        ))
        db.create_unique('software_version_machines', ['softwareversion_id', 'machine_id'])

        # Adding model 'SoftwareLicense'
        db.create_table('software_license', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('package', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['software.SoftwarePackage'])),
            ('version', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('date', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('text', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('software', ['SoftwareLicense'])

        # Adding model 'SoftwareLicenseAgreement'
        db.create_table('software_license_agreement', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.Person'])),
            ('license', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['software.SoftwareLicense'])),
            ('date', self.gf('django.db.models.fields.DateField')()),
        ))
        db.send_create_signal('software', ['SoftwareLicenseAgreement'])

    def backwards(self, orm):
        # Deleting model 'SoftwareCategory'
        db.delete_table('software_category')

        # Deleting model 'SoftwarePackage'
        db.delete_table('software_package')

        # Deleting model 'SoftwareVersion'
        db.delete_table('software_version')

        # Removing M2M table for field machines on 'SoftwareVersion'
        db.delete_table('software_version_machines')

        # Deleting model 'SoftwareLicense'
        db.delete_table('software_license')

        # Deleting model 'SoftwareLicenseAgreement'
        db.delete_table('software_license_agreement')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'machines.machine': {
            'Meta': {'object_name': 'Machine', 'db_table': "'machine'"},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['machines.MachineCategory']"}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mem_per_core': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'no_cpus': ('django.db.models.fields.IntegerField', [], {}),
            'no_nodes': ('django.db.models.fields.IntegerField', [], {}),
            'pbs_server_host': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'machines.machinecategory': {
            'Meta': {'object_name': 'MachineCategory', 'db_table': "'machine_category'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'people.institute': {
            'Meta': {'object_name': 'Institute', 'db_table': "'institute'"},
            'active_delegate': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'active_delegate'", 'null': 'True', 'to': "orm['people.Person']"}),
            'delegate': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'delegate'", 'null': 'True', 'to': "orm['people.Person']"}),
            'gid': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'sub_delegates': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'sub_delegates'", 'null': 'True', 'symmetrical': 'False', 'to': "orm['people.Person']"})
        },
        'people.person': {
            'Meta': {'object_name': 'Person', 'db_table': "'person'"},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'approved_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_approver'", 'null': 'True', 'to': "orm['people.Person']"}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'date_approved': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'deleted_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_deletor'", 'null': 'True', 'to': "orm['people.Person']"}),
            'department': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Institute']"}),
            'last_usage': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'mobile': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '8', 'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'supervisor': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'telephone': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'software.softwarecategory': {
            'Meta': {'object_name': 'SoftwareCategory', 'db_table': "'software_category'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'software.softwarelicense': {
            'Meta': {'object_name': 'SoftwareLicense', 'db_table': "'software_license'"},
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'package': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['software.SoftwarePackage']"}),
            'text': ('django.db.models.fields.TextField', [], {}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'software.softwarelicenseagreement': {
            'Meta': {'object_name': 'SoftwareLicenseAgreement', 'db_table': "'software_license_agreement'"},
            'date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'license': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['software.SoftwareLicense']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']"})
        },
        'software.softwarepackage': {
            'Meta': {'object_name': 'SoftwarePackage', 'db_table': "'software_package'"},
            'academic_only': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['software.SoftwareCategory']"}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'gid': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'homepage': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'tutorial_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'software.softwareversion': {
            'Meta': {'object_name': 'SoftwareVersion', 'db_table': "'software_version'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'machines': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['machines.Machine']", 'symmetrical': 'False'}),
            'module': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'package': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['software.SoftwarePackage']"}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['software']



# Copyright 2014-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import optparse

from django.core.management.base import BaseCommand
import django.db.transaction
import tldap.transaction
from tldap.dn import dn2str, str2dn
from tldap.schemas.rfc import organizationalUnit

from karaage.people.models import Person
from karaage.machines.models import Account, MachineCategory
from karaage.datastores import get_kg27_datastore
from karaage.datastores import get_global_test_datastore
from karaage.datastores import get_machine_category_test_datastore
import karaage.datastores.ldap as dldap


def _eq(dn1, dn2):
    dn1 = dn2str(str2dn(dn1)).lower()
    dn2 = dn2str(str2dn(dn2)).lower()
    return dn1 == dn2


class Command(BaseCommand):
    help = "Run migrations on the LDAP database."

    option_list = BaseCommand.option_list + (
        optparse.make_option(
            '--dry-run',
            action='store_true',
            dest='dry_run',
            default=False,
            help='Don\'t change anything, output what needs '
            'to change instead.'),
        optparse.make_option(
            '--delete',
            action='store_true',
            dest='delete',
            default=False,
            help='Delete records that are no longer used.'),
    )

    def get_base(self, datastore, key):
        if key in datastore._settings:
            base_dn = datastore._settings[key]
        else:
            connection = tldap.connections[datastore._using]
            base_dn = connection.settings_dict[key]
        return base_dn

    def create_base(self, datastore, create_dn):
        base_dn = dn2str(str2dn(create_dn)[1:])

        try:
            organizationalUnit.objects.using(
                using=datastore._using, settings=datastore._settings) \
                .base_dn(base_dn) \
                .get(dn=create_dn)
        except organizationalUnit.DoesNotExist:
            print("Creating %s" % create_dn)
            organizationalUnit.objects.using(
                using=datastore._using, settings=datastore._settings) \
                .base_dn(base_dn) \
                .create(dn=create_dn)

    @django.db.transaction.atomic
    @tldap.transaction.commit_on_success
    def handle(self, **options):
        delete = options['delete']
        dry_run = options['dry_run']

        self.handle_global(delete, dry_run)

        # recurse through every datastore
        for mc in MachineCategory.objects.all():
            print("")
            print("Processing %s" % mc)
            print("==========================================================")
            self.handle_machine_category(mc, delete, dry_run)

    def handle_global(self, delete, dry_run):
        try:
            global_datastore = get_global_test_datastore(0)
            assert isinstance(global_datastore, dldap.GlobalDataStore)
        except IndexError:
            global_datastore = None

        if global_datastore is not None:
            # we need to keep the people
            person_base_dn = self.get_base(
                global_datastore, 'LDAP_PERSON_BASE')
            pgroup_base_dn = self.get_base(
                global_datastore, 'LDAP_GROUP_BASE')
        else:
            # we need to destroy the people and keep the accounts
            person_base_dn = None
            pgroup_base_dn = None

        # create the base dn
        # note we do this even if --dry-run given, as otherwise
        # we get confused if dn doesn't exist
        if global_datastore is not None:
            self.create_base(global_datastore, person_base_dn)
            self.create_base(global_datastore, pgroup_base_dn)

    def handle_machine_category(self, mc, delete, dry_run):
        # if datastore name is not ldap, we are not interested
        if mc.datastore != "ldap":
            print(
                "machine category %s datastore %s is not LDAP; nothing to do"
                % (mc, mc.datastore))
            return

        # retreive the LDAP datastores
        kg27_datastore = get_kg27_datastore()
        if kg27_datastore is None:
            print("KG27_DATASTORE not set; nothing to do")
            return
        try:
            global_datastore = get_global_test_datastore(0)
            assert isinstance(global_datastore, dldap.GlobalDataStore)
        except IndexError:
            global_datastore = None
        machine_category_datastore = get_machine_category_test_datastore(
            mc.datastore, 0)
        assert isinstance(
            machine_category_datastore, dldap.MachineCategoryDataStore)

        # get the base dn
        kg27_account_dn = self.get_base(kg27_datastore, 'LDAP_ACCOUNT_BASE')
        kg27_group_dn = self.get_base(kg27_datastore, 'LDAP_GROUP_BASE')

        if global_datastore is not None:
            # we need to keep the people
            person_base_dn = self.get_base(
                global_datastore, 'LDAP_PERSON_BASE')
            pgroup_base_dn = self.get_base(
                global_datastore, 'LDAP_GROUP_BASE')
        else:
            # we need to destroy the people and keep the accounts
            person_base_dn = None
            pgroup_base_dn = None

        account_base_dn = self.get_base(
            machine_category_datastore, 'LDAP_ACCOUNT_BASE')
        agroup_base_dn = self.get_base(
            machine_category_datastore, 'LDAP_GROUP_BASE')

        # create the base dn
        # note we do this even if --dry-run given, as otherwise
        # we get confused if dn doesn't exist
        self.create_base(machine_category_datastore, account_base_dn)
        self.create_base(machine_category_datastore, agroup_base_dn)

        # sanity check
        assert pgroup_base_dn != agroup_base_dn
        assert person_base_dn != account_base_dn

        # process kg27 people/accounts
        for p in kg27_datastore._accounts() \
                .base_dn(kg27_account_dn):

            delete_this = delete

            # if this is an account, copy to new place
            if 'posixAccount' in p.objectClass:
                # this was an account; then there was no person in LDAP

                # copy account to correct place
                try:
                    dst = machine_category_datastore._accounts().get(uid=p.uid)
                    if _eq(p.dn, dst.dn):
                        delete_this = False
                    if 'posixAccount' not in dst.objectClass:
                        delete_this = False

                        # This shouldn't normally ever happen.
                        if dry_run:
                            print(
                                "Conflicting person exists, would delete "
                                "person %s " % dst.dn)
                        else:
                            print(
                                "Conflicting person exists, deleting "
                                "person %s " % dst.dn)
                            dst.delete()
                        raise machine_category_datastore._person.DoesNotExist
                except machine_category_datastore._account.DoesNotExist:
                    new_account = machine_category_datastore._create_account()
                    for i, _ in new_account.get_fields():
                        if i != "objectClass":
                            value = getattr(p, i)
                            setattr(new_account, i, value)

                    if dry_run:
                        print(
                            "Would copy account %s to account %s"
                            % (p.dn, account_base_dn))
                    else:
                        new_account.save()
                        print(
                            "Copying account %s to account %s"
                            % (p.dn, new_account.dn))

            # all kg27 entries are persons, copy person to correct place
            if global_datastore is not None:
                if 'posixAccount' in p.objectClass:
                    # we are looking at an account, we need to get the person
                    account = Account.objects.get(
                        username=p.uid, machine_category=mc)
                    person = Person.objects.get(account=account)
                    uid = person.username
                else:
                    # we are looking at a person, we already have the uid
                    uid = p.uid

                # Create person, if required.  This is better then calling
                # person.save() as we get the password too.
                try:
                    dst = global_datastore._people().get(uid=uid)
                    if _eq(p.dn, dst.dn):
                        delete_this = False
                    if 'posixAccount' in dst.objectClass:
                        delete_this = False
                        if dry_run:
                            print(
                                "Conflicting account exists, would delete "
                                "account %s " % dst.dn)
                        else:
                            print(
                                "Conflicting account exists, deleting "
                                "account %s " % dst.dn)
                            dst.delete()
                        raise global_datastore._person.DoesNotExist

                except global_datastore._person.DoesNotExist:
                    new_person = global_datastore._create_person()
                    for i, _ in new_person.get_fields():
                        if i != "objectClass":
                            value = getattr(p, i)
                            setattr(new_person, i, value)

                    if dry_run:
                        print(
                            "Would copy account %s to person %s"
                            % (p.dn, person_base_dn))
                    else:
                        new_person.save()
                        print(
                            "Copying account %s to person %s"
                            % (p.dn, new_person.dn))

            if delete_this:
                if dry_run:
                    print("Would delete %s" % p.dn)
                else:
                    print("deleting %s" % p.dn)
                    p.delete()

        # process groups
        for g in kg27_datastore._groups() \
                .base_dn(kg27_group_dn):

            delete_this = delete

            if global_datastore is not None:
                try:
                    dst = global_datastore._groups().get(cn=g.cn)
                    if _eq(g.dn, dst.dn):
                        delete_this = False
                except global_datastore._group.DoesNotExist:
                    new_group = global_datastore._create_group()
                    for i, _ in new_group.get_fields():
                        if i != "objectClass":
                            value = getattr(g, i)
                            setattr(new_group, i, value)

                    if dry_run:
                        print("Would copy group %s to %s"
                              % (g.dn, pgroup_base_dn))
                    else:
                        new_group.save()
                        print("Copying group %s to %s" % (g.dn, new_group.dn))

            try:
                dst = machine_category_datastore._groups().get(cn=g.cn)
                if _eq(g.dn, dst.dn):
                    delete_this = False
            except machine_category_datastore._group.DoesNotExist:
                new_group = machine_category_datastore._create_group()
                for i, _ in new_group.get_fields():
                    if i != "objectClass":
                        value = getattr(g, i)
                        setattr(new_group, i, value)

                if dry_run:
                    print("Would copy group %s to %s" % (g.dn, agroup_base_dn))
                else:
                    new_group.save()
                    print("Copying group %s to %s" % (g.dn, new_group.dn))

            if delete_this:
                if dry_run:
                    print("Would delete %s" % g.dn)
                else:
                    print("deleting %s" % g.dn)
                    g.delete()



# Copyright 2008, 2010-2011, 2013-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import patterns, url

urlpatterns = patterns(
    'karaage.institutes.views',
    url(r'^$', 'institute_list', name='kg_institute_list'),
    url(r'^add/$', 'add_edit_institute', name='kg_institute_add'),

    url(r'^quota/(?P<institutequota_id>\d+)/$',
        'institutequota_edit', name='kg_institutequota_edit'),
    url(r'^quota/(?P<institutequota_id>\d+)/delete/$',
        'institutequota_delete', name='kg_institutequota_delete'),

    url(r'^(?P<institute_id>\d+)/$',
        'institute_detail', name='kg_institute_detail'),
    url(r'^(?P<institute_id>\d+)/verbose/$',
        'institute_verbose', name='kg_institute_verbose'),
    url(r'^(?P<institute_id>\d+)/edit/$',
        'add_edit_institute', name='kg_institute_edit'),
    url(r'^(?P<institute_id>\d+)/quota/add/$',
        'institutequota_add', name='kg_institutequota_add'),
    url(r'^(?P<institute_id>[-.\w]+)/logs/$',
        'institute_logs', name='kg_institute_logs'),
    url(r'^(?P<institute_id>[-.\w]+)/add_comment/$',
        'add_comment', name='kg_institute_add_comment'),

)

profile_urlpatterns = patterns(
    'karaage.institutes.views',
    url(r'^institutes/$', 'profile_institutes', name='kg_profile_institutes'),
)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration
from django.db import connection


class Migration(SchemaMigration):

    # Migrations that must run *after* we update the person model and remove the
    # user FK.
    needed_by = (
        ('machines', '0013_nop'),
        ('projects', '0010_auto'),
        ('institutes', '0006_auto__chg_field_institutechunk_institute'),
        ('software', '0009_nop'),
        ('common', '0002_move_comments'),
        ('applications', '0019_auto__add_field_applicant_short_name__add_field_applicant_full_name'),
    )

    def forwards(self, orm):
        cursor = connection.cursor()
        django_comments_exists = ('django_comments' in
                                  connection.introspection.get_table_list(cursor))
        django_comment_flags_exists = ('django_comment_flags' in
                                       connection.introspection.get_table_list(cursor))

        db.alter_column(
            'django_admin_log', 'person_id',
            self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person']))

        if django_comments_exists:
            db.alter_column('django_comments', 'person_id',
                            self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person']))
        if django_comment_flags_exists:
            db.alter_column('django_comment_flags', 'person_id',
                            self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person']))

        db.delete_column('django_admin_log', 'user_id')
        if django_comments_exists:
            db.delete_column('django_comments', 'user_id')
        if django_comment_flags_exists:
            db.delete_column('django_comment_flags', 'user_id')

        db.rename_column('django_admin_log', 'person_id', 'user_id')
        if django_comments_exists:
            db.rename_column('django_comments', 'person_id', 'user_id')
        if django_comment_flags_exists:
            db.rename_column('django_comment_flags', 'person_id', 'user_id')

        # Deleting field 'Person.user'
        db.delete_column('person', 'user_id')

        # Changing field 'Person.username'
        db.alter_column('person', 'username', self.gf('django.db.models.fields.CharField')(unique=True, max_length=30))

        # Changing field 'Person.full_name'
        db.alter_column('person', 'full_name', self.gf('django.db.models.fields.CharField')(max_length=60))

        # Changing field 'Person.short_name'
        db.alter_column('person', 'short_name', self.gf('django.db.models.fields.CharField')(max_length=30))

    def backwards(self, orm):
        cursor = connection.cursor()
        django_comments_exists = ('django_comments' in
                                  connection.introspection.get_table_list(cursor))
        django_comment_flags_exists = ('django_comment_flags' in
                                       connection.introspection.get_table_list(cursor))

        db.rename_column('django_admin_log', 'user_id', 'person_id')
        if django_comments_exists:
            db.rename_column('django_comments', 'user_id', 'person_id')
        if django_comment_flags_exists:
            db.rename_column('django_comment_flags', 'user_id', 'person_id')

        db.add_column('django_admin_log', 'user',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User']),
                      keep_default=False)
        if django_comments_exists:
            db.add_column('django_comments', 'user',
                          self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User']),
                          keep_default=False)
        if django_comment_flags_exists:
            db.add_column('django_comment_flags', 'user',
                          self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User']),
                          keep_default=False)

        db.alter_column(
            'django_admin_log', 'person_id',
            self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person'], null=True))
        if django_comments_exists:
            db.alter_column('django_comments', 'person_id',
                            self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person'], null=True))
        if django_comment_flags_exists:
            db.alter_column('django_comment_flags', 'person_id',
                            self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person'], null=True))

        # Adding field 'Person.user'
        db.add_column('person', 'user',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], unique=True, null=True),
                      keep_default=False)

        # Changing field 'Person.username'
        db.alter_column('person', 'username', self.gf('django.db.models.fields.CharField')(unique=True, max_length=30, null=True))

        # Changing field 'Person.full_name'
        db.alter_column('person', 'full_name', self.gf('django.db.models.fields.CharField')(max_length=60, null=True))

        # Changing field 'Person.short_name'
        db.alter_column('person', 'short_name', self.gf('django.db.models.fields.CharField')(max_length=30, null=True))

    models = {
        'institutes.institute': {
            'Meta': {'ordering': "['name']", 'object_name': 'Institute', 'db_table': "'institute'"},
            'delegates': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'delegate'", 'to': "orm['people.Person']", 'through': "orm['institutes.InstituteDelegate']", 'blank': 'True', 'symmetrical': 'False', 'null': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'saml_entityid': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'institutes.institutedelegate': {
            'Meta': {'object_name': 'InstituteDelegate', 'db_table': "'institutedelegate'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['institutes.Institute']"}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']"}),
            'send_email': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'people.group': {
            'Meta': {'ordering': "['name']", 'object_name': 'Group'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'groups'", 'symmetrical': 'False', 'to': "orm['people.Person']"}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        },
        'people.person': {
            'Meta': {'ordering': "['full_name', 'short_name']", 'object_name': 'Person', 'db_table': "'person'"},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'approved_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_approver'", 'null': 'True', 'to': "orm['people.Person']"}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True', 'blank': 'True'}),
            'date_approved': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'deleted_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_deletor'", 'null': 'True', 'to': "orm['people.Person']"}),
            'department': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'db_index': 'True'}),
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'full_name': ('django.db.models.fields.CharField', [], {'max_length': '60'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['institutes.Institute']"}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_admin': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_systemuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_usage': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'legacy_ldap_password': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'login_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mobile': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '8', 'null': 'True', 'blank': 'True'}),
            'saml_id': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'short_name': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'supervisor': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'telephone': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['people']



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Group'
        db.create_table('people_group', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('people', ['Group'])

        # Adding M2M table for field members on 'Group'
        db.create_table('people_group_members', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('group', models.ForeignKey(orm['people.group'], null=False)),
            ('person', models.ForeignKey(orm['people.person'], null=False))
        ))
        db.create_unique('people_group_members', ['group_id', 'person_id'])

    def backwards(self, orm):
        # Deleting model 'Group'
        db.delete_table('people_group')

        # Removing M2M table for field members on 'Group'
        db.delete_table('people_group_members')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'institutes.institute': {
            'Meta': {'ordering': "['name']", 'object_name': 'Institute', 'db_table': "'institute'"},
            'delegates': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'delegate'", 'to': "orm['people.Person']", 'through': "orm['institutes.InstituteDelegate']", 'blank': 'True', 'symmetrical': 'False', 'null': 'True'}),
            'gid': ('django.db.models.fields.IntegerField', [], {}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'saml_entityid': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'institutes.institutedelegate': {
            'Meta': {'object_name': 'InstituteDelegate'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['institutes.Institute']"}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']"}),
            'send_email': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'people.group': {
            'Meta': {'object_name': 'Group'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'groups'", 'symmetrical': 'False', 'to': "orm['people.Person']"}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        },
        'people.person': {
            'Meta': {'ordering': "['first_name', 'last_name']", 'object_name': 'Person', 'db_table': "'person'"},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'approved_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_approver'", 'null': 'True', 'to': "orm['people.Person']"}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True', 'blank': 'True'}),
            'date_approved': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'deleted_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_deletor'", 'null': 'True', 'to': "orm['people.Person']"}),
            'department': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['institutes.Institute']"}),
            'is_systemuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_usage': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'login_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mobile': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '8', 'null': 'True', 'blank': 'True'}),
            'saml_id': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'supervisor': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'telephone': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['people']



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    depends_on = (
        ("admin", "0004_auto__del_logentry"),
        ("applications", "0028_auto__chg_field_applicant_username"),
        ("cache", "0015_auto__del_taskmachinecategorycache__del_unique_taskmachinecategorycach"),
        ("common", "0007_fix_project_logs"),
        ("emails", "0002_auto__del_emailtemplate"),
        ("institutes", "0008_auto__chg_field_institute_name"),
        ("machines", "0021_auto__chg_field_account_username"),
        ("pbsmoab", "0001_initial"),
        ("people", "0024_auto__chg_field_person_username"),
        ("projects", "0018_auto__chg_field_project_pid"),
        ("software", "0013_rename_user_to_person"),
        ("usage", "0013_auto__del_taskcacheforproject__del_unique_taskcacheforproject_date_sta"),
    )

    def forwards(self, orm):
        if not db.dry_run:
            apps = ['admin', 'cache', 'common', 'emails',
                    'institutes', 'machines', 'pbsmoab', 'people', 'projects',
                    ]
            orm['contenttypes.contenttype'].objects \
                .filter(app_label__in=apps).update(app_label='karaage')

    def backwards(self, orm):
        pass

    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['contenttypes']



# Copyright 2008, 2010, 2013-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import patterns, url

urlpatterns = patterns(
    'karaage.people.views.persons',
    url(r'^$', 'user_detail', name='kg_person_detail'),
    url(r'^verbose/$', 'user_verbose', name='kg_person_verbose'),
    url(r'^activate/$', 'activate', name='kg_person_activate'),
    url(r'^delete/$', 'delete_user', name='kg_person_delete'),
    url(r'^password/$', 'password_change', name='kg_person_password'),
    url(r'^lock/$', 'lock_person', name='kg_person_lock'),
    url(r'^unlock/$', 'unlock_person', name='kg_person_unlock'),
    url(r'^bounced_email/$', 'bounced_email', name='kg_person_bounce'),
    url(r'^logs/$', 'person_logs', name='kg_person_logs'),
    url(r'^add_comment/$', 'add_comment', name='kg_person_add_comment'),
    url(r'^edit/$', 'edit_user', name='kg_person_edit'),
    url(r'^password_request/$', 'password_request', name='kg_person_reset'),
    url(r'^password_request/done/$',
        'password_request_done', name='kg_person_reset_done'),
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import connection


class Migration(SchemaMigration):
    # Migrations that must run *before* we update the person model and remove the
    # user FK.

    depends_on = (
        ('machines', '0012_rename_useraccount_to_account'),
        ('projects', '0009_auto__del_field_project_machine_category'),
        ('institutes', '0005_auto__del_field_institute_gid'),
        ('software', '0008_auto__del_field_software_gid'),
        ('common', '0001_initial'),
        ('admin', '00001_initial_create'),
        ('applications', '0018_auto__add_unique_applicant_saml_id'),
    )

    def forwards(self, orm):
        db.add_column('django_admin_log', 'person',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person'], null=True),
                      keep_default=False)

        # If the django comments table exists then add the extra missing fields
        cursor = connection.cursor()
        if 'django_comments' in connection.introspection.get_table_list(cursor):
            db.add_column('django_comments', 'person',
                          self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person'], null=True),
                          keep_default=False)
        if 'django_comment_flags' in connection.introspection.get_table_list(cursor):
            db.add_column('django_comment_flags', 'person',
                          self.gf('django.db.models.fields.related.ForeignKey')(to=orm['people.person'], null=True),
                          keep_default=False)

        # Adding field 'Person.password'
        db.add_column('person', 'password',
                      self.gf('django.db.models.fields.CharField')(default='INVALID', max_length=128),
                      keep_default=False)

        # Adding field 'Person.last_login'
        db.add_column('person', 'last_login',
                      self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now),
                      keep_default=False)

        # Adding field 'Person.username'
        db.add_column('person', 'username',
                      self.gf('django.db.models.fields.CharField')(max_length=30, unique=True, null=True),
                      keep_default=False)

        # Adding field 'Person.email'
        db.add_column('person', 'email',
                      self.gf('django.db.models.fields.EmailField')(max_length=75, null=True, db_index=True),
                      keep_default=False)

        # Adding field 'Person.short_name'
        db.add_column('person', 'short_name',
                      self.gf('django.db.models.fields.CharField')(max_length=30, null=True),
                      keep_default=False)

        # Adding field 'Person.full_name'
        db.add_column('person', 'full_name',
                      self.gf('django.db.models.fields.CharField')(max_length=60, null=True),
                      keep_default=False)

        # Adding field 'Person.is_active'
        db.add_column('person', 'is_active',
                      self.gf('django.db.models.fields.BooleanField')(default=True),
                      keep_default=False)

        # Adding field 'Person.is_admin'
        db.add_column('person', 'is_admin',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

        # Changing field 'Person.user'
        db.alter_column('person', 'user_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], unique=True, null=True))

    def backwards(self, orm):
        db.delete_column('django_admin_log', 'person_id')

        cursor = connection.cursor()
        if 'django_comments' in connection.introspection.get_table_list(cursor):
            db.delete_column('django_comments', 'person_id')
        if 'django_comment_flags' in connection.introspection.get_table_list(cursor):
            db.delete_column('django_comment_flags', 'person_id')

        # Deleting field 'Person.password'
        db.delete_column('person', 'password')

        # Deleting field 'Person.last_login'
        db.delete_column('person', 'last_login')

        # Deleting field 'Person.username'
        db.delete_column('person', 'username')

        # Deleting field 'Person.email'
        db.delete_column('person', 'email')

        # Deleting field 'Person.short_name'
        db.delete_column('person', 'short_name')

        # Deleting field 'Person.full_name'
        db.delete_column('person', 'full_name')

        # Deleting field 'Person.is_active'
        db.delete_column('person', 'is_active')

        # Deleting field 'Person.is_admin'
        db.delete_column('person', 'is_admin')

        # Changing field 'Person.user'
        db.alter_column('person', 'user_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], unique=True))

    models = {
        'admin.logentry': {
            'Meta': {'ordering': "('-action_time',)", 'object_name': 'LogEntry', 'db_table': "u'django_admin_log'"},
            'action_flag': ('django.db.models.fields.PositiveSmallIntegerField', [], {}),
            'action_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'change_message': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'object_repr': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']", 'null': True}),
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'comments.comment': {
            'Meta': {'ordering': "('submit_date',)", 'object_name': 'Comment', 'db_table': "'django_comments'"},
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '3000'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'content_type_set_for_comment'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_removed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'object_pk': ('django.db.models.fields.TextField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'submit_date': ('django.db.models.fields.DateTimeField', [], {'default': 'None'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'comment_comments'", 'null': 'True', 'to': "orm['auth.User']"}),
            'user_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'user_name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'user_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']", 'null': True}),
        },
        'comments.commentflag': {
            'Meta': {'unique_together': "[('user', 'comment', 'flag')]", 'object_name': 'CommentFlag', 'db_table': "'django_comment_flags'"},
            'comment': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'flags'", 'to': "orm['comments.Comment']"}),
            'flag': ('django.db.models.fields.CharField', [], {'max_length': '30', 'db_index': 'True'}),
            'flag_date': ('django.db.models.fields.DateTimeField', [], {'default': 'None'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'comment_flags'", 'to': "orm['auth.User']"}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']", 'null': True}),
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'institutes.institute': {
            'Meta': {'ordering': "['name']", 'object_name': 'Institute', 'db_table': "'institute'"},
            'delegates': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'delegate'", 'to': "orm['people.Person']", 'through': "orm['institutes.InstituteDelegate']", 'blank': 'True', 'symmetrical': 'False', 'null': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'saml_entityid': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'institutes.institutedelegate': {
            'Meta': {'object_name': 'InstituteDelegate', 'db_table': "'institutedelegate'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['institutes.Institute']"}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']"}),
            'send_email': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'people.group': {
            'Meta': {'ordering': "['name']", 'object_name': 'Group'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'groups'", 'symmetrical': 'False', 'to': "orm['people.Person']"}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        },
        'people.person': {
            'Meta': {'ordering': "['full_name', 'short_name']", 'object_name': 'Person', 'db_table': "'person'"},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'approved_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_approver'", 'null': 'True', 'to': "orm['people.Person']"}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True', 'blank': 'True'}),
            'date_approved': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'deleted_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_deletor'", 'null': 'True', 'to': "orm['people.Person']"}),
            'department': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'db_index': 'True'}),
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'full_name': ('django.db.models.fields.CharField', [], {'max_length': '60', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['institutes.Institute']"}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_admin': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_systemuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_usage': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'legacy_ldap_password': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'login_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mobile': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '8', 'null': 'True', 'blank': 'True'}),
            'saml_id': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'short_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'supervisor': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'telephone': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True', 'null': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True', 'null': 'True'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['people', 'admin', 'comments']



# -*- coding: utf-8 -*-
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        pass

    def backwards(self, orm):
        pass

    models = {
    }

    complete_apps = ['cache']



# Copyright 2010-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

"""
Management utility to create superusers.
"""

import getpass
import os
import sys
from optparse import make_option
import re

from django.core import exceptions
from django.core.management.base import BaseCommand
from django.core.validators import validate_email
from django.conf import settings

from karaage.people.models import Person, Group
from karaage.institutes.models import Institute
from karaage.people.utils import validate_username_for_new_person
from karaage.people.utils import UsernameException

import django.db.transaction
import tldap.transaction

try:
    input = raw_input
except NameError:
    pass


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--username', '-u', help='Username'),
        make_option('--email', '-e', help='E-Mail'),
        make_option('--short_name', '-f', help='Short Name'),
        make_option('--full_name', '-l', help='Full Name'),
        make_option('--password', '-p', help='Password'),
        make_option('--institute', '-i', help='Institute Name'),
    )
    help = 'Used to create a karaage superuser.'

    @django.db.transaction.atomic
    @tldap.transaction.commit_on_success
    def handle(self, *args, **options):

        username = options['username']
        email = options['email']
        short_name = options['short_name']
        full_name = options['full_name']
        password = options['password']
        institute_name = options['institute']

        # Try to determine the current system user's username to use as a
        # default.
        try:
            import pwd
            unix_uid = os.getuid()
            unix_username = pwd.getpwuid(unix_uid)[0]
            default_username = unix_username.replace(' ', '').lower()
            if default_username == 'root':
                default_username = ''
        except (ImportError, KeyError):
            # KeyError will be raised by getpwuid() if there is no
            # corresponding entry in the /etc/passwd file (a very restricted
            # chroot environment, for example).
            default_username = ''

        # Determine whether the default username is taken, so we don't display
        # it as an option.
        if default_username:
            try:
                Person.objects.get(username=default_username)
            except Person.DoesNotExist:
                pass
            else:
                default_username = ''

        # Prompt for username/email/password. Enclose this whole thing in a
        # try/except to trap for a keyboard interrupt and exit gracefully.
        try:
            # Get a username
            while 1:
                if not username:
                    input_msg = 'Username'
                    if default_username:
                        input_msg += \
                            ' (Leave blank to use %r)' % default_username
                    username = input(input_msg + ': ')
                    if default_username and username == '':
                        username = default_username
                try:
                    validate_username_for_new_person(username)
                    break
                except UsernameException as e:
                    sys.stderr.write("%s\n" % e)
                    username = None
                    print('')
                    continue

            # Get an email
            while 1:
                if not email:
                    email = input('E-mail address: ')
                try:
                    validate_email(email)
                except exceptions.ValidationError:
                    sys.stderr.write(
                        "Error: That e-mail address is invalid.\n")
                    print('')
                    email = None
                else:
                    break

            # Get a password
            while 1:
                if not password:
                    password = getpass.getpass()
                    password2 = getpass.getpass('Password (again): ')
                    if password != password2:
                        sys.stderr.write(
                            "Error: Your passwords didn't match.\n")
                        password = None
                        print('')
                        continue
                if password.strip() == '':
                    sys.stderr.write(
                        "Error: Blank passwords aren't allowed.\n")
                    password = None
                    print('')
                    continue
                break

            while 1:
                if not short_name:
                    short_name = input('Short Name: ')
                else:
                    break
            while 1:
                if not full_name:
                    full_name = input('Full Name: ')
                else:
                    break

            group_re = re.compile(r'^%s$' % settings.GROUP_VALIDATION_RE)
            while 1:
                if not institute_name:
                    if Institute.objects.count() > 0:
                        print(
                            "Choose an existing institute "
                            "for new superuser.")
                        print("Alternatively enter a new name to create one.")
                        print("")
                        print("Valid choices are:")
                        print("")
                        for i in Institute.active.all():
                            print("* %s" % i)
                        print
                    else:
                        print("No Institutes in system, will create one now.")
                        print('')

                    institute_name = input('Institute Name: ')

                if not re.search(group_re, institute_name):
                    sys.stderr.write(
                        "%s\n" % settings.GROUP_VALIDATION_ERROR_MSG)
                    institute_name = None
                    print('')
                    continue
                else:
                    break

            try:
                institute = Institute.objects.get(name=institute_name)
                print("Using existing institute %s." % institute)

            except Institute.DoesNotExist:
                group, c = Group.objects.get_or_create(name=institute_name)
                if c:
                    print("Created new group %s." % group)
                else:
                    print("Using existing group %s." % group)

                institute = Institute.objects.create(
                    name=institute_name, group=group, is_active=True)

                print("Created new institute %s." % institute)

        except KeyboardInterrupt:
            sys.stderr.write("\nOperation cancelled.\n")
            sys.exit(1)

        data = {
            'username': username,
            'email': email,
            'password': password,
            'short_name': short_name,
            'full_name': full_name,
            'institute': institute,
        }
        Person.objects.create_superuser(**data)
        print("Karaage Superuser created successfully.")



# Copyright 2010, 2014-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.conf.urls import patterns, url

profile_urlpatterns = patterns(
    'karaage.common.views.profile',
    url(r'^$', 'profile', name='kg_profile'),
    url(r'^logout/$', 'logout', name='kg_profile_logout'),
)

profile_urlpatterns += patterns(
    'karaage.people.views.profile',
    url(r'^personal/$', 'profile_personal', name='kg_profile_personal'),
    url(r'^edit/$', 'edit_profile', name='kg_profile_edit'),
    url(r'^password/$', 'password_change', name='kg_profile_password'),
    url(r'^password_request/$', 'password_request', name='kg_profile_reset'),
    url(r'^password_request/done/$',
        'password_request_done', name='kg_profile_reset_done'),
    url(r'^login/$', 'login', name='kg_profile_login'),
    url(r'^login/(?P<username>%s)/$' % settings.USERNAME_VALIDATION_RE,
        'login', name="kg_profile_login"),
)

if settings.SHIB_SUPPORTED:
    profile_urlpatterns += patterns(
        'karaage.people.views.profile',
        url(r'^saml/$', 'saml_details', name='kg_profile_saml'),
        url(r'^slogin/$', 'saml_login', name='kg_profile_login_saml'),
    )



# Copyright 2009-2011, 2013-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.template import RequestContext
from django.core.mail import send_mass_mail
from django.core.urlresolvers import reverse
from django.contrib import messages

from karaage.common.decorators import admin_required
from karaage.emails.forms import BulkEmailForm


@admin_required
def send_email(request):

    form = BulkEmailForm(request.POST or None)
    if request.method == 'POST':

        if form.is_valid():
            if 'preview' in request.POST:
                emails = form.get_emails()
                try:
                    preview = emails[0]
                except IndexError:
                    pass
            else:
                send_mass_mail(form.get_emails())
                messages.success(request, "Emails sent successfully")

                return HttpResponseRedirect(reverse('index'))

    return render_to_response(
        'karaage/emails/send_email_form.html', locals(),
        context_instance=RequestContext(request))



# Copyright 2008, 2010-2011, 2013-2015 VPAC
# Copyright 2014 The University of Melbourne
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import datetime
import importlib
import warnings
import six

import django
from django.http import QueryDict
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType

from karaage.plugins import BasePlugin
from karaage.middleware.threadlocals import get_current_user
from karaage.common.forms import CommentForm
from karaage.common.models import (LogEntry, ADDITION, CHANGE,
                                   DELETION, COMMENT)


def get_date_range(request, default_start=None, default_end=None):

    if default_start is None:
        default_start = datetime.date.today() - datetime.timedelta(days=90)
    if default_end is None:
        default_end = datetime.date.today()

    today = datetime.date.today()

    if 'start' in request.GET:
        try:
            years, months, days = request.GET['start'].split('-')
            start = datetime.datetime(int(years), int(months), int(days))
            start = start.date()
        except:
            start = today - datetime.timedelta(days=90)
    else:
        start = default_start

    if 'end' in request.GET:
        try:
            years, months, days = request.GET['end'].split('-')
            end = datetime.datetime(int(years), int(months), int(days))
            end = end.date()
        except:
            end = today
    else:
        end = default_end

    return start, end


def get_current_person():
    user = get_current_user()
    if user is None:
        return None
    if not user.is_authenticated():
        return None
    return user


class log():

    def __init__(self, user, obj, flag, message):
        warnings.warn("Calling karaage.common.log directly has been"
                      " deprecated. You should use the API "
                      "log.(add|change|field_change|delete|comment)",
                      DeprecationWarning)
        LogEntry.objects.log_object(obj, flag, message, user)

    @classmethod
    def add(cls, obj, message, user=None):
        return LogEntry.objects.log_object(obj, ADDITION, message, user)

    @classmethod
    def change(cls, obj, message, user=None):
        return LogEntry.objects.log_object(obj, CHANGE, message, user)

    @classmethod
    def field_change(cls, obj, user=None, field=None, new_value=None):
        return LogEntry.objects.log_object(
            obj, CHANGE, 'Changed %s to %s' % (field, new_value), user)

    @classmethod
    def delete(cls, obj, message, user=None):
        return LogEntry.objects.log_object(obj, DELETION, message, user)

    @classmethod
    def comment(cls, obj, message, user=None):
        return LogEntry.objects.log_object(obj, COMMENT, message, user)


def new_random_token():
    import random
    from hashlib import sha1
    # Use the system (hardware-based) random number generator if it exists.
    if hasattr(random, 'SystemRandom'):
        randrange = random.SystemRandom().randrange
    else:
        randrange = random.randrange
    max_key = 18446744073709551616     # 2 << 63

    string = six.u("%s%s") % (randrange(0, max_key), settings.SECRET_KEY)
    return sha1(string.encode("ascii")).hexdigest()


def log_list(request, breadcrumbs, obj):
    result = QueryDict("", mutable=True)
    result['content_type'] = ContentType.objects.get_for_model(obj).pk
    result['object_id'] = obj.pk
    url = reverse('kg_log_list') + "?" + result.urlencode()
    return HttpResponseRedirect(url)


def add_comment(request, breadcrumbs, obj):
    assert obj is not None
    assert obj.pk is not None
    form = CommentForm(
        data=request.POST or None, obj=obj,
        request=request, instance=None)
    if request.method == 'POST':
        form.save()
        return HttpResponseRedirect(obj.get_absolute_url())

    return render_to_response(
        'karaage/common/add_comment.html',
        {
            'form': form, 'obj': obj,
            'breadcrumbs': breadcrumbs,
        },
        context_instance=RequestContext(request))


def is_admin(request):
    if settings.ADMIN_IGNORED:
        return False
    if not request.user.is_authenticated():
        return False
    return request.user.is_admin


def get_app_modules(name):
    if django.VERSION < (1, 7):
        for app in settings.INSTALLED_APPS:
            try:
                module_name = app + "." + name
                module = importlib.import_module(module_name)
                yield module
            except ImportError:
                pass
    else:
        from django.apps import apps
        for config in apps.get_app_configs():
            if isinstance(config, BasePlugin):
                module_name = config.name + "." + name
                module = importlib.import_module(module_name)
                yield module


def get_urls(name):
    for module in get_app_modules("urls"):
        urls = getattr(module, name, None)
        if urls is not None:
            yield urls



# Copyright 2008-2011, 2013-2015 VPAC
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

""" Common hooks for all datastores. """

import six
import importlib
import types

from django.conf import settings

from .base import GlobalDataStore, MachineCategoryDataStore
from karaage.machines.models import MachineCategory

_GLOBAL_DATASTORES = []
_MACHINE_CATEGORY_DATASTORES = {}
_KG27_DATASTORE = None


def _lookup(cls):
    """ Lookup module.class. """
    if isinstance(cls, str):
        module_name, _, name = cls.rpartition(".")
        module = importlib.import_module(module_name)
        try:
            cls = getattr(module, name)
        except AttributeError:
            raise AttributeError("%s reference cannot be found" % cls)
    return cls


def _get_datastore(cls, expected, config):
    assert isinstance(cls, types.FunctionType) or issubclass(cls, expected)
    ds = cls(config)
    assert isinstance(ds, expected)
    return ds


def _init_datastores():
    """ Initialize all datastores. """
    global _GLOBAL_DATASTORES
    global _MACHINE_CATEGORY_DATASTORES
    global _KG27_DATASTORE
    array = settings.GLOBAL_DATASTORES
    for config in array:
        cls = _lookup(config['ENGINE'])
        ds = _get_datastore(cls, GlobalDataStore, config)
        _GLOBAL_DATASTORES.append(ds)
    for name, array in six.iteritems(settings.MACHINE_CATEGORY_DATASTORES):
        _MACHINE_CATEGORY_DATASTORES[name] = []
        for config in array:
            cls = _lookup(config['ENGINE'])
            ds = _get_datastore(cls, MachineCategoryDataStore, config)
            _MACHINE_CATEGORY_DATASTORES[name].append(ds)
    if settings.KG27_DATASTORE is not None:
        config = settings.KG27_DATASTORE
        cls = _lookup(config['ENGINE'])
        ds = _get_datastore(cls, MachineCategoryDataStore, config)
        _KG27_DATASTORE = ds


def _get_global_datastores():
    global _GLOBAL_DATASTORES
    return _GLOBAL_DATASTORES


def _get_machine_category_datastores(machine_category):
    """ Get the datastores for machine category. """
    global _MACHINE_CATEGORY_DATASTORES
    name = machine_category.datastore
    return _MACHINE_CATEGORY_DATASTORES[name]


def _get_machine_categorys():
    return MachineCategory.objects.all()


def get_global_test_datastore(number=None):
    """ For testing only. Do not use. """
    global _GLOBAL_DATASTORES
    if number is None:
        number = settings.LDAP_TEST_DATASTORE_N
    datastores = _GLOBAL_DATASTORES
    return datastores[number]


def get_machine_category_test_datastore(name=None, number=None):
    """ For testing only. Do not use. """
    global _MACHINE_CATEGORY_DATASTORES
    if name is None:
        name = settings.LDAP_TEST_DATASTORE
    if number is None:
        number = settings.LDAP_TEST_DATASTORE_N
    datastores = _MACHINE_CATEGORY_DATASTORES[name]
    return datastores[number]


def get_kg27_datastore():
    """ For upgrade only. Do not use. """
    global _KG27_DATASTORE
    return _KG27_DATASTORE

# Initialize data stores
_init_datastores()


######################################################################
# GLOBAL DATA STORES                                                 #
######################################################################


##########
# Person #
##########

def global_save_person(person):
    """ Person was saved. """
    for datastore in _get_global_datastores():
        datastore.save_person(person)


def global_delete_person(person):
    """ Person was deleted. """
    for datastore in _get_global_datastores():
        datastore.delete_person(person)


def global_set_person_password(person, raw_password):
    """ Person's password was changed. """
    for datastore in _get_global_datastores():
        datastore.set_person_password(person, raw_password)


def global_set_person_username(person, old_username, new_username):
    """ Person's username was changed. """
    for datastore in _get_global_datastores():
        datastore.set_person_username(person, old_username, new_username)


def global_add_person_to_group(person, group):
    """ Add person to group. """
    for datastore in _get_global_datastores():
        datastore.add_person_to_group(person, group)


def global_remove_person_from_group(person, group):
    """ Remove person from group. """
    for datastore in _get_global_datastores():
        datastore.remove_person_from_group(person, group)


def global_add_person_to_project(person, project):
    """ Add person to project. """
    for datastore in _get_global_datastores():
        datastore.add_person_to_project(person, project)


def global_remove_person_from_project(person, project):
    """ Remove person from project. """
    for datastore in _get_global_datastores():
        datastore.remove_person_from_project(person, project)


def global_person_exists(username):
    """ Does this person exist??? """
    for datastore in _get_global_datastores():
        if datastore.person_exists(username):
            return True
    return False


def global_get_person_details(person):
    """ Get details for this user. """
    result = []
    for datastore in _get_global_datastores():
        value = datastore.get_person_details(person)
        value['datastore'] = datastore.config['DESCRIPTION']
        result.append(value)
    return result


#########
# Group #
#########

def global_save_group(group):
    """ Group was saved. """
    for datastore in _get_global_datastores():
        datastore.save_group(group)


def global_delete_group(group):
    """ Group was deleted. """
    for datastore in _get_global_datastores():
        datastore.delete_group(group)


def global_set_group_name(group, old_name, new_name):
    """ Group was renamed. """
    for datastore in _get_global_datastores():
        datastore.set_group_name(group, old_name, new_name)


def global_get_group_details(group):
    """ Get group details. """
    result = []
    for datastore in _get_global_datastores():
        value = datastore.get_group_details(group)
        value['datastore'] = datastore.config['DESCRIPTION']
        result.append(value)
    return result


######################################################################
# MACHINE CATEGORY DATASTORES                                        #
######################################################################


###########
# Account #
###########

def machine_category_save_account(account):
    """ Account was saved. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.save_account(account)


def machine_category_delete_account(account):
    """ Account was deleted. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.delete_account(account)


def machine_category_set_account_password(account, raw_password):
    """ Account's password was changed. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.set_account_password(account, raw_password)


def machine_category_set_account_username(account, old_username, new_username):
    """ Account's username was changed. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.set_account_username(account, old_username, new_username)


def machine_category_add_account_to_group(account, group):
    """ Add account to group. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.add_account_to_group(account, group)


def machine_category_remove_account_from_group(account, group):
    """ Remove account from group. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.remove_account_from_group(account, group)


def machine_category_add_account_to_project(account, project):
    """ Add account to project. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.add_account_to_project(account, project)


def machine_category_remove_account_from_project(account, project):
    """ Remove account from project. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.remove_account_from_project(account, project)


def machine_category_add_account_to_institute(account, institute):
    """ Add account to institute. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.add_account_to_institute(account, institute)


def machine_category_remove_account_from_institute(account, institute):
    """ Remove account from institute. """
    for datastore in _get_machine_category_datastores(
            account.machine_category):
        datastore.remove_account_from_institute(account, institute)


def machine_category_account_exists(username, machine_category):
    """ Does the account exist??? """
    for datastore in _get_machine_category_datastores(machine_category):
        if datastore.account_exists(username):
            return True
    return False


def machine_category_get_account_details(account):
    """ Get the account details. """
    machine_category = account.machine_category
    result = {machine_category.name: []}
    for datastore in _get_machine_category_datastores(machine_category):
        value = datastore.get_account_details(account)
        value['datastore'] = datastore.config['DESCRIPTION']
        result[machine_category.name].append(value)
    return result


#########
# Group #
#########

def machine_category_save_group(group):
    """ Group was saved. """
    for machine_category in _get_machine_categorys():
        for datastore in _get_machine_category_datastores(machine_category):
            datastore.save_group(group)


def machine_category_delete_group(group):
    """ Group was deleted. """
    for machine_category in _get_machine_categorys():
        for datastore in _get_machine_category_datastores(machine_category):
            datastore.delete_group(group)


def machine_category_set_group_name(group, old_name, new_name):
    """ Group was renamed. """
    for machine_category in _get_machine_categorys():
        for datastore in _get_machine_category_datastores(machine_category):
            datastore.set_group_name(group, old_name, new_name)


def machine_category_get_group_details(group):
    """ Get group details. """
    result = {}
    for machine_category in _get_machine_categorys():
        result[machine_category.name] = []
        for datastore in _get_machine_category_datastores(machine_category):
            value = datastore.get_group_details(group)
            value['datastore'] = datastore.config['DESCRIPTION']
            result[machine_category.name].append(value)
    return result


###########
# Project #
###########

def machine_category_save_project(project):
    """ An machine has been saved. """
    for project_quota in project.projectquota_set.all():
        for datastore in _get_machine_category_datastores(
                project_quota.machine_category):
            datastore.save_project(project)


def machine_category_delete_project(project, machine_category=None):
    """ An machine has been deleted. """
    if machine_category is None:
        for project_quota in project.projectquota_set.all():
            for datastore in _get_machine_category_datastores(
                    project_quota.machine_category):
                datastore.delete_project(project)
    else:
        for datastore in _get_machine_category_datastores(machine_category):
            datastore.delete_project(project)


def machine_category_get_project_details(project):
    """ Get details for this user. """
    result = {}
    for project_quota in project.projectquota_set.all():
        machine_category = project_quota.machine_category
        result[machine_category.name] = []
        for datastore in _get_machine_category_datastores(machine_category):
            value = datastore.get_project_details(project)
            value['datastore'] = datastore.config['DESCRIPTION']
            result[machine_category.name].append(value)
    return result


def machine_category_set_project_pid(project, old_pid, new_pid):
    """ Project's PID was changed. """
    for project_quota in project.projectquota_set.all():
        for datastore in _get_machine_category_datastores(
                project_quota.machine_category):
            datastore.save_project(project)
            datastore.set_project_pid(project, old_pid, new_pid)


#############
# Institute #
#############

def machine_category_save_institute(institute):
    """ An institute has been saved. """
    for institute_quota in institute.institutequota_set.all():
        for datastore in _get_machine_category_datastores(
                institute_quota.machine_category):
            datastore.save_institute(institute)


def machine_category_delete_institute(institute, machine_category=None):
    """ An institute has been deleted. """
    if machine_category is None:
        for institute_quota in institute.institutequota_set.all():
            for datastore in _get_machine_category_datastores(
                    institute_quota.machine_category):
                datastore.delete_institute(institute)
    else:
        for datastore in _get_machine_category_datastores(machine_category):
            datastore.delete_institute(institute)


def machine_category_get_institute_details(institute):
    """ Get details for this user. """
    result = {}
    for institute_quota in institute.institutequota_set.all():
        machine_category = institute_quota.machine_category
        result[machine_category.name] = []
        for datastore in _get_machine_category_datastores(machine_category):
            value = datastore.get_institute_details(institute)
            value['datastore'] = datastore.config['DESCRIPTION']
            result[machine_category.name].append(value)
    return result


######################################################################
# OTHER                                                              #
######################################################################

def add_accounts_to_group(accounts_query, group):
    """ Add accounts to group. """

    query = accounts_query.filter(date_deleted__isnull=True)

    for account in query:
        machine_category_add_account_to_group(account, group)


def remove_accounts_from_group(accounts_query, group):
    """ Remove accounts from group. """

    query = accounts_query.filter(date_deleted__isnull=True)

    for account in query:
        machine_category_remove_account_from_group(account, group)


def add_accounts_to_project(accounts_query, project):
    """ Add accounts to project. """

    query = accounts_query.filter(date_deleted__isnull=True)
    query = query.filter(machine_category__projectquota__project=project)

    for account in query:
        machine_category_add_account_to_project(account, project)


def remove_accounts_from_project(accounts_query, project):
    """ Remove accounts from project. """

    query = accounts_query.filter(date_deleted__isnull=True)
    query = query.filter(machine_category__projectquota__project=project)

    for account in query:
        machine_category_remove_account_from_project(account, project)


def add_accounts_to_institute(accounts_query, institute):
    """ Add accounts to institute. """

    query = accounts_query.filter(date_deleted__isnull=True)
    query = query.filter(machine_category__institutequota__institute=institute)

    for account in query:
        machine_category_add_account_to_institute(account, institute)


def remove_accounts_from_institute(accounts_query, institute):
    """ Remove accounts from institute. """

    query = accounts_query.filter(date_deleted__isnull=True)
    query = query.filter(machine_category__institutequota__institute=institute)

    for account in query:
        machine_category_remove_account_from_institute(account, institute)


###################
# MachineCategory #
###################

def set_mc_datastore(machine_category, old_datastore, new_datastore):
    from karaage.people.models import Group
    from karaage.machines.models import Account, MachineCategory

    mc_query = None
    if new_datastore is not None:
        mc_query = MachineCategory.objects
        mc_query = mc_query.filter(datastore=new_datastore)
        mc_query = mc_query.exclude(pk=machine_category.pk)
        other_mc_refer_datastore = (mc_query.count() > 0)
        for datastore in _get_machine_category_datastores(new_datastore):
            if not other_mc_refer_datastore:
                for group in Group.objects.all():
                    datastore.save_group(group)
            for account in Account.objects.filter(
                    date_deleted__isnull=True,
                    machine_category=machine_category):
                datastore.save_account(account)



# -*- coding: utf-8 -*-
from south.v2 import DataMigration
from karaage.datastores import get_kg27_datastore


class Migration(DataMigration):

    def forwards(self, orm):
        datastore = get_kg27_datastore()
        if datastore is None:
            return
        for account in orm['machines.useraccount'].objects.filter(date_deleted__isnull=True):
            try:
                p = datastore._accounts().get(uid=account.username)
                if p.is_locked():
                    account.shell = account.previous_shell
                else:
                    account.shell = p.loginShell
                if not account.shell:
                    account.shell = "/bin/bash"
                account.save()
            except datastore._account.DoesNotExist:
                print(
                    "+++ account username=%s has no ldap entry; "
                    "can't set account shell"
                    % account.username)
                pass

    def backwards(self, orm):
        for account in orm['machines.useraccount'].objects.iterator():
            account.previous_shell = account.shell
            account.save()

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'machines.machine': {
            'Meta': {'object_name': 'Machine', 'db_table': "'machine'"},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['machines.MachineCategory']"}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mem_per_core': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'no_cpus': ('django.db.models.fields.IntegerField', [], {}),
            'no_nodes': ('django.db.models.fields.IntegerField', [], {}),
            'pbs_server_host': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'scaling_factor': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'start_date': ('django.db.models.fields.DateField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'machines.machinecategory': {
            'Meta': {'object_name': 'MachineCategory', 'db_table': "'machine_category'"},
            'datastore': ('django.db.models.fields.CharField', [], {'default': "'karaage.datastores.openldap_datastore'", 'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'machines.useraccount': {
            'Meta': {'ordering': "['user']", 'object_name': 'UserAccount', 'db_table': "'user_account'"},
            'date_created': ('django.db.models.fields.DateField', [], {}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'default_project': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['projects.Project']", 'null': 'True', 'blank': 'True'}),
            'disk_quota': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'machine_category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['machines.MachineCategory']"}),
            'previous_shell': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'shell': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']"}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'people.institute': {
            'Meta': {'ordering': "['name']", 'object_name': 'Institute', 'db_table': "'institute'"},
            'delegates': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'delegate'", 'to': "orm['people.Person']", 'through': "orm['people.InstituteDelegate']", 'blank': 'True', 'symmetrical': 'False', 'null': 'True'}),
            'gid': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'saml_entityid': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'people.institutedelegate': {
            'Meta': {'object_name': 'InstituteDelegate'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Institute']"}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Person']"}),
            'send_email': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'people.person': {
            'Meta': {'object_name': 'Person', 'db_table': "'person'"},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'approved_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_approver'", 'null': 'True', 'to': "orm['people.Person']"}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True', 'blank': 'True'}),
            'date_approved': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'deleted_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_deletor'", 'null': 'True', 'to': "orm['people.Person']"}),
            'department': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Institute']"}),
            'is_systemuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_usage': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'login_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mobile': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '8', 'null': 'True', 'blank': 'True'}),
            'saml_id': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'supervisor': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'telephone': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'projects.project': {
            'Meta': {'ordering': "['pid']", 'object_name': 'Project', 'db_table': "'project'"},
            'additional_req': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'approved_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'project_approver'", 'null': 'True', 'to': "orm['people.Person']"}),
            'date_approved': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_deleted': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'deleted_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'project_deletor'", 'null': 'True', 'to': "orm['people.Person']"}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'institute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['people.Institute']"}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_approved': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_usage': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'leaders': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'leaders'", 'symmetrical': 'False', 'to': "orm['people.Person']"}),
            'machine_categories': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'projects'", 'null': 'True', 'symmetrical': 'False', 'to': "orm['machines.MachineCategory']"}),
            'machine_category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['machines.MachineCategory']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'pid': ('django.db.models.fields.CharField', [], {'max_length': '50', 'primary_key': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'default': 'datetime.datetime(2013, 5, 17, 0, 0)'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['people.Person']", 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['machines']
    symmetrical = True
